use sea_orm_migration::prelude::*;
use crate::m20230619_221344_create_user::User;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(Setting::Table)
                    .if_not_exists()
                    .col(
                        ColumnDef::new(Setting::Id)
                            .string()
                            .not_null()
                            .primary_key(),
                    )
                    .col(ColumnDef::new(Setting::Timestamp).date_time().not_null())
                    //
                    // Instance metadata
                    .col(
                        ColumnDef::new(Setting::Domain)
                            .string()
                            .not_null()
                    )
                    .col(
                        ColumnDef::new(Setting::Name)
                            .string()
                            .not_null()
                            .default("Slop"),
                    )
                    .col(ColumnDef::new(Setting::Description).string())
                    .col(ColumnDef::new(Setting::ExtendedDescription).string())
                    .col(ColumnDef::new(Setting::PrivacyPolicy).string())
                    // The following 2 columns will later get foreign keys when files are impl
                    .col(ColumnDef::new(Setting::Icon).string())
                    .col(ColumnDef::new(Setting::Thumbnail).string())
                    //
                    // Contact/Support info
                    .col(ColumnDef::new(Setting::ContactUser).string())
                    .col(ColumnDef::new(Setting::ContactEmail).string())
                    //
                    // Instance privacy
                    .col(
                        ColumnDef::new(Setting::ShowDomainBlocks)
                            .boolean()
                            .not_null()
                            .default(false),
                    )
                    .col(
                        ColumnDef::new(Setting::ShowDomainBlockReason)
                            .boolean()
                            .not_null()
                            .default(false),
                    )
                    .col(
                        ColumnDef::new(Setting::RegistrationsEnabled)
                            .boolean()
                            .not_null()
                            .default(false),
                    )
                    .col(
                        ColumnDef::new(Setting::RegistrationsRequireApproval)
                            .boolean()
                            .not_null()
                            .default(true),
                    )
                    .col(ColumnDef::new(Setting::RegistrationsDisabledMessage).boolean())
                    .col(
                        ColumnDef::new(Setting::InvitesEnabled)
                            .boolean()
                            .not_null()
                            .default(true),
                    )
                    .col(
                        ColumnDef::new(Setting::PublishStatistics)
                            .boolean()
                            .not_null()
                            .default(false),
                    )
                    //
                    // Limits
                    .col(
                        ColumnDef::new(Setting::MaxPostCharacters)
                            .big_unsigned()
                            .not_null()
                            .default(2000),
                    )
                    .col(
                        ColumnDef::new(Setting::MaxPostAttachments)
                            .big_unsigned()
                            .not_null()
                            .default(5),
                    )
                    .col(
                        ColumnDef::new(Setting::ImageSizeLimit)
                            .big_unsigned()
                            .not_null()
                            .default(10485760),
                    )
                    .col(
                        ColumnDef::new(Setting::VideoSizeLimit)
                            .big_unsigned()
                            .not_null()
                            .default(16777216),
                    )
                    .col(
                        ColumnDef::new(Setting::AvatarSizeLimit)
                            .big_unsigned()
                            .not_null()
                            .default(10485760),
                    )
                    .col(
                        ColumnDef::new(Setting::BannerSizeLimit)
                            .big_unsigned()
                            .not_null()
                            .default(10485760),
                    )
                    //
                    // Foreign Keys
                    .foreign_key(
                        ForeignKey::create()
                            .name("fk-setting_contact_user-user_id")
                            .from(Setting::Table, Setting::ContactUser)
                            .to(User::Table, User::Id)
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(Setting::Table).to_owned())
            .await
    }
}

/// Learn more at https://docs.rs/sea-query#iden
#[derive(Iden)]
enum Setting {
    Table,
    Id,
    Timestamp,

    // Instance metadata
    /// The domain of the instance
    Domain,
    /// Name of the instance
    Name,
    /// Description of the instance
    Description,
    /// A much longer "extended" description, formatted with markdown
    ExtendedDescription,
    /// The privacy policy for the instance, formatted with markdown
    PrivacyPolicy,
    /// Object file id pointing to the icon
    Icon,
    /// Object file id pointing to a 2:1 "thumbnail" image
    Thumbnail,

    // Contact/Support info
    /// Main central "support" account for the instance
    ContactUser,
    /// An email for off-site support and/or legal requests
    ContactEmail,

    // Instance privacy
    /// Show (publicly) blocked domains
    ShowDomainBlocks,
    /// Show (publicly) the reasoning behind blocked domains
    ShowDomainBlockReason,
    /// Enable or disable registration for the instance
    RegistrationsEnabled,
    /// Require approval by staff when registering
    RegistrationsRequireApproval,
    /// Message to show when registrations are disabled
    RegistrationsDisabledMessage,
    InvitesEnabled,
    /// Publish instance statistics to the API (in weekly increments)
    /// This includes:
    /// * Locally published posts
    /// * Active users
    /// * New users
    PublishStatistics,

    // Limits
    /// Max amount of characters a post can have
    MaxPostCharacters,
    /// Max amount of attachments (images and videos) a post can have
    MaxPostAttachments,
    /// Max file size for an image uploaded as an attachment
    ImageSizeLimit,
    /// Max file size for a video uploaded as an attachment
    VideoSizeLimit,
    /// Maximum file size for a user's avatar
    AvatarSizeLimit,
    /// Maximum file size for a user's banner
    BannerSizeLimit,
}
